<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Tco00600GerenteNegocio extends \Teknisa\GER\Controllers\TeknisaCrud{
    protected $environment;
    protected $entityManager;
    protected $crudService;
    protected $tco00600GerenteNegocioService;
    
    public function __construct($crudService, $parameterBag, $environment, $entityManager, $tco00600GerenteNegocioService) {
        $this->environment                    = $environment;
        $this->entityManager                  = $entityManager;
        $this->crudService                    = $crudService;
        $this->tco00600GerenteNegocioService  = $tco00600GerenteNegocioService;
    }

    /**
     * Função responsável por adicionar operadores
     * como gerentes caso estejam ativos. A função utiliza
     * uma verificação para evitar que o newCode ou o 
     * CDOPERADORLIDER sejam alterados no frontend.
     * 
     * @param {Request} $request Request do usuário.
     * @param {Response} $response Response da requisição.
     */
    public function saveManager(DTO\Request $request, DTO\Response $response){
        $dataSetRow = $request->getDataSet()->getRows()[0];
        $isInvalidNewManager = $this->isInvalidNewManager($dataSetRow);
        
        if(!$isInvalidNewManager){
            $this->tco00600GerenteNegocioService->saveManager($request, $response);
        } else {
            $response->setError(new Error($isInvalidNewManager, 1));
        }
    }

    /**
     * Função responsável por remover gerentes.
     * 
     * @param {Request} $request Request do usuário.
     * @param {Response} $response Response da requisição.
     */
    public function removeManager(DTO\Request $request, DTO\Response $response){
        $nrGerente = $request->getRow()->get('NRGERENTENEGOCIO');
        $invalidState = $this->isInvalidManagerRemove($nrGerente);
        
        if(!sizeof($invalidState)){
            $this->tco00600GerenteNegocioService->removeManager($request, $response);
        } else {
            $response->addDataSet(new DataSet('invalidManagerRemove', $invalidState));
        }
    }

    /**
     * Função responsável por verificar se o gerente 
     * selecionado é invalido para exclusão. Verifica
     * se possui algum gerente sob sua liderança, e se
     * não possui dependências em tabelas que usam 
     * gerente (NRGERENTENEGOCIO).
     * 
     * @param {String} $nrGerente Número do genrente
     *  selecionado.
     * 
     * @return {Array} Retorna array com dados de invalidez
     *  da exclusão do registro.
     */
    private function isInvalidManagerRemove($nrGerente){
        return $this->tco00600GerenteNegocioService->getDependencies($nrGerente);
    }

    /**
     * Função responsável por adicionar gerentes à 
     * liderança de outro.
     * 
     * @param {Request} $request Request do usuário.
     * @param {Response} $response Response da requisição.
     */
    public function addToManagerGrid(DTO\Request $request, DTO\Response $response){
        $cdLider = $request->getRow()->get('cdLider');
        $toAdd = $request->getRow()->get('toAdd');
        
        if(sizeof($toAdd)){
            $invalidManagers = $this->tco00600GerenteNegocioService->addToManagerGrid($cdLider, $toAdd);
            $response->addDataSet(new DataSet('invalidManagers', $invalidManagers));
        }
    }

    /**
     * Função responsável por remover gerentes da
     * liderança de outro.
     * 
     * @param {Request} $request Request do usuário.
     * @param {Response} $response Response da requisição.
     */
    public function removeFromManagerGrid(DTO\Request $request, DTO\Response $response){
        $cdLider = $request->getRow()->get('cdLider');
        $toRemove = $request->getRow()->get('toRemove');
        
        $this->tco00600GerenteNegocioService->removeFromManagerGrid($cdLider, $toRemove);
    }

    /**
     * Função responsável por verificar se o novo
     * registro de operador é inválido.
     * 
     * @param {DTO\Row} $dataSetRow Objeto row contendo
     *  os dados de adição.
     * 
     * @return {Boolean|String} Retorna false caso o
     *  registro não seja invalido, e o código do erro
     *  caso seja.
     */
    private function isInvalidNewManager($dataSetRow){
        $cdOperador = $dataSetRow->get('CDOPERADOR');
        $validState = false;
        
        if($this->tco00600GerenteNegocioService->isValidNewManagerDataSet($dataSetRow)){
            if(!$this->tco00600GerenteNegocioService->isActiveOperator($cdOperador))
                $validState = "TCO00600_OPERADOR_INATIVO";
        } else {
            $validState = "TCO00600_DADOS_INCONSISTENTES";
        }

        return $validState;
    }
    
    /**
     * Função responsável por alterar a liderança de gerentes.
     * 
     * @param {Request} $request Request do usuário.
     * @param {Response} $response Response da requisição.
     */
    public function changeManager(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $operatorsToUpdate = $this->getOperatorFromLeader($row['OLD_CDOPERADORLIDER']);
        
        $this->updateNewLeaderOperators($operatorsToUpdate, $row['CDOPERADOR']);
    }
    
    /**
     * Função responsável por retornar os gerentes com base
     * em seu lider.
     * 
     * @param {String} $cdoperadorlider String do código do Lider.
     */
    
    private function getOperatorFromLeader($cdoperadorlider) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT GN.CDOPERADOR,
                       OP.NMOPERADOR,
                       GN.CDOPERADORLIDER,
                       LIDER.NMOPERADOR NMOPERADORLIDER,
                       GN.NRGERENTENEGOCIO,
                       GN.IDATIVO,
                       GN.NRORG
                  FROM GERENTENEGOCIO GN
                 INNER JOIN OPERADOR OP
                    ON GN.CDOPERADOR = OP.CDOPERADOR
                  LEFT JOIN OPERADOR LIDER
                    ON GN.CDOPERADORLIDER = LIDER.CDOPERADOR
                 WHERE GN.CDOPERADORLIDER = :CDOPERADORLIDER
                   AND GN.NRORG = :NRORG
                 ORDER BY GN.CDOPERADOR";
                   
        $query = $connection->prepare($sql);
        
        $params = array(
            "CDOPERADORLIDER" => $cdoperadorlider,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        return $result;
    }
    
    /**
     * Função responsável por alterar o lider dos gerentes.
     * 
     * @param {Array} $operatorsToUpdate Array de gerentes.
     * @param {String} $cdoperadorlider String do código do Lider.
     */
    
    private function updateNewLeaderOperators($operatorsToUpdate, $cdoperadorlider) {
        
        $rowsToUpdate = array();
        
        foreach($operatorsToUpdate as $row) {
            $row['CDOPERADORLIDER'] = $cdoperadorlider;
            $row['__is_new'] = false;
            $row['changed'] = true;
            array_push($rowsToUpdate, $row);
        }
        
        $dataSet = new DataSet("gerentenegocio", $rowsToUpdate);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rows = $this->crudService->getRowsToSave($request, "gerentenegocio");
        $this->crudService->save("gerentenegocio", $rows);
    }
}